/**
 *    author:  tourist
 *    created: 13.10.2023 01:55:12       
**/
#include <bits/stdc++.h>

using namespace std;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

constexpr long double pi = acos((long double) -1.0);
constexpr long double eps = static_cast<long double>(1e-9);

template <typename T>
struct TPoint {
  T x;
  T y;
  int id;

  TPoint() : x(0), y(0), id(-1) {}
  TPoint(const T& x_, const T& y_) : x(x_), y(y_), id(-1) {}
  TPoint(const T& x_, const T& y_, int id_) : x(x_), y(y_), id(id_) {}

  inline TPoint operator+(const TPoint& rhs) const { return TPoint(x + rhs.x, y + rhs.y); }
  inline TPoint operator-(const TPoint& rhs) const { return TPoint(x - rhs.x, y - rhs.y); }
  inline TPoint operator*(const T& rhs) const { return TPoint(x * rhs, y * rhs); }
  inline TPoint operator/(const T& rhs) const { return TPoint(x / rhs, y / rhs); }

  friend T smul(const TPoint& a, const TPoint& b) {
    return a.x * b.x + a.y * b.y;
  }

  friend T vmul(const TPoint& a, const TPoint& b) {
    return a.x * b.y - a.y * b.x;
  }

  inline T abs2() const {
    return x * x + y * y;
  }

  inline bool operator<(const TPoint& rhs) const {
    return (y < rhs.y || (y == rhs.y && x < rhs.x));
  }

  inline bool is_upper() const {
    return (y > eps || (abs(y) <= eps && x > eps));
  }

  inline int cmp_polar(const TPoint& rhs) const {
    assert(abs(x) > eps || abs(y) > eps);
    assert(abs(rhs.x) > eps || abs(rhs.y) > eps);
    bool a = is_upper();
    bool b = rhs.is_upper();
    if (a != b) {
      return (a ? -1 : 1);
    }
    long double v = x * rhs.y - y * rhs.x;
    return (v > eps ? -1 : (v < -eps ? 1 : 0));
  }
};

//using Point = TPoint<long long>;
using Point = TPoint<long double>;

template <typename T>
string to_string(const TPoint<T>& p) {
  return "(" + to_string(p.x) + ", " + to_string(p.y) + ")";
}

int main() {
  ios::sync_with_stdio(false);
  cin.tie(0);
  int n;
  cin >> n;
  vector<Point> p(n);
  for (int i = 0; i < n; i++) {
    cin >> p[i].x >> p[i].y;
  }
  auto mid = Point(0, 0);
  for (int i = 0; i < n; i++) {
    mid = mid + p[i];
  }
  mid = mid / n;
  for (int i = 0; i < n; i++) {
    p[i] = p[i] - mid;
  }
  sort(p.begin(), p.end(), [&](Point a, Point b) {
    return a.cmp_polar(b) == -1;
  });
  long double area = 0;
  long double per = 0;
  for (int i = 0; i < n; i++) {
    int j = (i + 1) % n;
    area += (p[i].x - p[j].x) * (p[i].y + p[j].y);
    per += sqrt((p[i] - p[j]).abs2());
  }
  area = abs(area) * 0.5;
  vector<long double> thr(100);
  auto goal = area;
  for (int it = 1; it < 100; it++) {
    goal *= 2;
    long double low = 0, high = 1e30;
    for (int iter = 0; iter < 200; iter++) {
      auto md = (low + high) * 0.5;
      auto got = area + pi * md * md + per * md;
      if (got >= goal) {
        high = md;
      } else {
        low = md;
      }
    }
    thr[it] = 0.5 * (low + high);
  }
  int tt;
  cin >> tt;
  while (tt--) {
    Point pt;
    cin >> pt.x >> pt.y;
    pt = pt - mid;
    if (abs(pt.x) <= eps && abs(pt.y) <= eps) {
      cout << 0 << '\n';
      continue;
    }
    int low = -1, high = n;
    while (low + 1 < high) {
      int md = (low + high) >> 1;
      if (p[md].cmp_polar(pt) == -1) {
        low = md;
      } else {
        high = md;
      }
    }
    {
      int i1 = (low + n) % n;
      int i2 = (i1 + 1) % n;
      auto a = p[i2].y - p[i1].y;
      auto b = p[i1].x - p[i2].x;
      auto c = -a * p[i1].x - b * p[i1].y;
      if (a * pt.x + b * pt.y + c <= eps) {
        cout << 0 << '\n';
        continue;
      }
    }
    int lft = -1;
    int rgt = -1;
    {
      int L = low - n + 2, R = low;
      while (L < R) {
        int M = L + (R - L) / 2;
        int i = (M % n + n) % n;
        if (vmul(pt, p[i]) >= 0) {
          L = M + 1;
          continue;
        }
        if (vmul(p[(i + 1) % n] - p[i], pt - p[i]) >= 0) {
          L = M + 1;
        } else {
          R = M;
        }
      }
      lft = L;
    }
    {
      int L = high, R = high + n - 2;
      while (L < R) {
        int M = L + (R - L + 1) / 2;
        int i = (M % n + n) % n;
        if (vmul(p[i], pt) >= 0) {
          R = M - 1;
          continue;
        }
        if (vmul(p[(i + n - 1) % n] - p[i], pt - p[i]) <= 0) {
          R = M - 1;
        } else {
          L = M;
        }
      }
      rgt = L;
    }
    while (lft < rgt) {
      int center = lft + (rgt - lft) / 2;
      auto d1 = (p[(center % n + n) % n] - pt).abs2();
      auto d2 = (p[(center % n + n + 1) % n] - pt).abs2();
      if (d1 < d2) {
        rgt = center;
      } else {
        lft = center + 1;
      }
    }
    int i = (lft % n + n) % n;
    auto di = (p[i] - pt).abs2();
    auto ans = di;
    for (int delta = -1; delta <= 1; delta += 2) {
      int j = (i + delta + n) % n;
      auto dj = (p[j] - pt).abs2();
      auto dij = (p[j] - p[i]).abs2();
      if (dj > di + dij) {
        continue;
      }
      auto a = p[j].y - p[i].y;
      auto b = p[i].x - p[j].x;
      auto c = -a * p[i].x - b * p[i].y;
      auto d = a * pt.x + b * pt.y + c;
      d *= d;
      d /= a * a + b * b;
      ans = min(ans, d);
    }
    ans = sqrt(ans);
    for (int k = 1; ; k++) {
      if (ans <= thr[k]) {
        cout << k << '\n';
        break;
      }
    }
  }
  return 0;
}
